--------------------------------------------------------------------------------
-- Copyright  2021								|
-- All Rights Reserved.								|
-- 										|
-- Permission to use, copy, modify, and distribute this software in any way	|
-- is not granted without expressed written agreement. This copyright applies	|
-- to the all code, data and artwork related to this project and all associated	|
-- projects published by the author.						|
--										|
-- More information on copyright is available from sources on-line.		|
-- Use an actual factual one!							|
--------------------------------------------------------------------------------

local AddonName, NS = ...

local CheckSize, ColourSize, FontSize, XOfffset = 22, 19, 14, 2

local FontPath = "Interface\\Addons\\"..AddonName.."\\Fonts\\"
local TexturePath = "Interface\\Addons\\"..AddonName.."\\Config\\"

local floor = math.floor

local function CreateBackdrop(name, parent, ftype, bg, edge, totile, esize, tsize, l, r, t, b, strata)
	local backdrop = {
		bgFile = bg,
		edgeFile = edge,
		tile = totile,
		edgeSize = eSize,
		tileSize = tSize,
		insets = {
			left = l,
			right = r,
			top = t,
			bottom = b,
		},
	}
	local f = CreateFrame(ftype, name, parent, BackdropTemplateMixin and "BackdropTemplate") -- BackdropTemplateMixin required for Classic
	f:SetFrameStrata(strata or "BACKGROUND")
	f:SetBackdrop(backdrop)
	f:SetBackdropColor(0, 0, 0, 0)
	return f
end

local function CreateTooltipBackdrop(name, parent, ftype)
	local backdrop = {
		bgFile = "Interface\\BUTTONS\\WHITE8X8",
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
		tile = true,
		edgeSize = 7,
		tileSize = 7,
		insets = {
			left = 2,
			right = 2,
			top = 2,
			bottom = 2,
		},
	}
	local f = CreateFrame(ftype, name, parent, BackdropTemplateMixin and "BackdropTemplate") -- BackdropTemplateMixin required for Classic
	f:SetFrameStrata("BACKGROUND") 
	f:SetBackdrop(backdrop)
	f:SetBackdropColor(0, 0, 0, 0)
	return f
end

local function EditBox_Init(self, data, index)
	self.data = data
	self.index = index
	local text = self.data[self.index]
	if not text then
		text = ""
	end
	self.prevvalue = text
	self:SetText(text)
end

local function EditBox_Update(self)
	local value = self:GetText()
	if (not value) then value = "" end
	if self.number then
		value = self:GetNumber()
		if not value then
			value = 0
			self:SetText("0")
		end
	end
	self:ClearFocus()
	self.data[self.index] = value
	if self.updatefunc then
		self.updatefunc(self)
	end
end

local function CheckBox_Init(self, data, index)
	self.data = data
	self.index = index
	self:SetChecked(self.data[self.index])
end

local function Slider_Init(self, data, index)
	self.data = data
	self.index = index
	local setting = self.data[self.index]
	if self.scale then
		setting = setting * self.scale
	end
	local min, max = self:GetMinMaxValues()
	if setting < min or setting > max then
		return
	end
	self:SetValue(setting)
end

local function Slider_Update(self)
	local setting = self.data[self.index]
	if self.scale then
		setting = setting * self.scale
	end
	local min, max = self:GetMinMaxValues()

	if setting and (setting < min or setting > max) then
		return
	end
	local value
	if self.scale then
		value = self:GetValue()
	else
		value = floor(self:GetValue())
	end
	value = floor(value)
	self.editbox:SetText(value)
	if self.scale then
		value = value / self.scale
	end
	self.data[self.index] = value
	if self.preupdatefunc then
		self.preupdatefunc()
	end
	NS:UpdateSettings()
end

local function Slider_UpdateFromEditBox(self)
	local parent = self:GetParent()
	local value = self:GetNumber()
	if not value then value = 0 end
	local min, max = parent:GetMinMaxValues()
	if parent.minlock and value < min then 
		value = min 
	end
	if parent.maxlock and value > max then 
		value = max 
	end
	value = floor(value)
	self:SetText(value)
	if value >= min and value <= max then
		parent:SetValue(value)
	end
	self:ClearFocus()
	if parent.scale then
		value = value / parent.scale
	end
	parent.data[parent.index] = value
	NS:UpdateSettings()
end

local function ColourPicker_Init(self, data, index)
	self.data = data 
	self.index = index
	local colour = self.data[self.index]
	self:SetBackdropColor(colour.r, colour.g, colour.b)
end

local function ColourPicker_Cancelled(self)
	local colour = ColorPickerFrame.previousValues
	local settings = ColorPickerFrame.colourBox.data[ColorPickerFrame.colourBox.index]
	settings.r = colour.r
	settings.g = colour.g
	settings.b = colour.b
	ColorPickerFrame.colourBox:SetBackdropColor(colour.r, colour.g, colour.b)
	if ColorPickerFrame.colourBox.updatefunc then
		ColorPickerFrame.colourBox.updatefunc(self)
	end
	NS:UpdateSettings()
end

local function ColourPicker_Changed(self)
	local r, g, b = ColorPickerFrame:GetColorRGB()
	local settings = ColorPickerFrame.colourBox.data[ColorPickerFrame.colourBox.index]
	settings.r = r
	settings.g = g
	settings.b = b
	ColorPickerFrame.colourBox:SetBackdropColor(r, g, b)
	if ColorPickerFrame.colourBox.updatefunc then
		ColorPickerFrame.colourBox.updatefunc(self)
	end
	NS:UpdateSettings()
end

local function ColourPicker_OnClick(self)
	if self.prehookfunc then
		self.prehookfunc(self)
	end
	local basecolour = self.data[self.index]
	local colour = {}
	colour.r = basecolour.r
	colour.g = basecolour.g
	colour.b = basecolour.b
	ColorPickerFrame.previousValues = colour
	ColorPickerFrame.cancelFunc = ColourPicker_Cancelled
	ColorPickerFrame.opacityFunc = ColourPicker_Changed
	ColorPickerFrame.func = ColourPicker_Changed
	ColorPickerFrame.colourBox = self
	ColorPickerFrame:SetColorRGB(colour.r, colour.g, colour.b)
	ColorPickerFrame:ClearAllPoints()
	if self:GetRight() < UIParent:GetWidth() / 2 then
		ColorPickerFrame:SetPoint("LEFT", self, "RIGHT", 10, 0)
	else
		ColorPickerFrame:SetPoint("RIGHT", self, "LEFT", -10, 0)
	end
	ColorPickerFrame:Show()
end

local function SetToolTip(self, tip, anchor)
	local setAnchor = anchor or "ANCHOR_LEFT"
	if self and tip then
		GameTooltip:SetOwner(self, setAnchor)
		GameTooltip:SetText(tip)			
	else
		GameTooltip:Hide()
	end
end

local function DropMenu_Init(self, data, index, table)--, override)
	self.table = table
	self.data = data
	self.index = index
	local text
	local initval = self.data[self.index]
	for _, value in pairs(self.table) do
		if value.value == initval then
			text = value.text
			break
		end
	end
	if (not text) and self.addoptions then
		for _, value in pairs(self.addoptions) do
			if value.value == initval then
				text = value.text
				break
			end
		end
	end
	if not text then
		text = ""
	end
	self.setting:SetText(text)
end

local function DropList_Show(self, toggle)
	local  parentcontrol = self
	if toggle then
		parentcontrol = self:GetParent()
	end
	local menu = parentcontrol.menu
	if menu:IsVisible() then
		menu:Hide()
		if menu.addedoptions then
			for i in pairs(menu.addedoptions) do
				parentcontrol.menu.table[i] = nil
			end
		end
		return
	end
	if parentcontrol.addoptions then
		menu.addedoptions = {}
		for i,v in pairs(parentcontrol.addoptions) do
			tinsert(parentcontrol.table, v)
			menu.addedoptions[#parentcontrol.table] = 1
		end
	else
		menu.addedoptions = nil
	end
	menu:ClearAllPoints()
	menu:SetPoint("TOPLEFT", parentcontrol, "BOTTOMLEFT", 0, 0)
	if menu:GetBottom() < UIParent:GetBottom() then
		local yoffset = UIParent:GetBottom() - menu:GetBottom()
		menu:ClearAllPoints()
		menu:SetPoint("TOPLEFT", parentcontrol.setting, "BOTTOMLEFT", 0, yoffset)
	end
	menu:Show()
	menu.table = parentcontrol.table
	menu.index = parentcontrol.index
	menu.subindex = parentcontrol.subindex
	menu.subindex2 = parentcontrol.subindex2
	menu.miscindex = parentcontrol.miscindex
	menu.controlbox = parentcontrol.setting
	if menu.scrollMenu then
		local numButtons = #menu.Buttons
		for i=1, numButtons do
			menu.Buttons[i]:SetChecked(0)
		end
		parentcontrol = menu.ScrollFrame
		local scroll = _G[parentcontrol:GetName().."ScrollBar"]
		scroll:SetValue(0)
		parentcontrol.offset = 0
		menu.updatefunc(parentcontrol)
	end
	if not menu.scrollMenu then
		local count = 0
		local widest = 0
		for _, value in pairs(parentcontrol.table) do
			count = count + 1
			parentcontrol.menu.drop[count]:Show()
			parentcontrol.menu.drop[count].text:SetText(value.text)
			parentcontrol.menu.drop[count].value = value.value
			parentcontrol.menu.drop[count].desc = value.desc
			local width = parentcontrol.menu.drop[count].text:GetWidth()
			if (width > widest) then
				widest = width
			end
		end
		for i=1, menu.count do
			if (i <= count) then
				parentcontrol.menu.drop[i]:SetWidth(widest)
			else
				parentcontrol.menu.drop[i]:Hide()			
			end
		end
		menu:SetWidth(widest + 20)
		menu:SetHeight(count * 15 + 20)
	end
	menu.trap:SetWidth(menu:GetWidth()+6)
	menu.trap:SetHeight(menu:GetHeight()+menu.trap.addheight)
end

local function ScrollMenu_Update(self)
	local parent = self:GetParent()
	local table = parent.table
	if not table or not parent.Buttons then return end
	local numOptions = #table
	local offset = FauxScrollFrame_GetOffset(self)
	if not offset then offset = 0; end
	local index, button
	local numButtons = #parent.Buttons
	local lastbutton
	for i=1, numButtons do
		button = parent.Buttons[i]
		buttontext = button.Text
		index = offset + i
		if table[index] then
			buttontext:SetText(table[index].text)
			button:Show()
			button:SetChecked(false)
			button.value = table[index].value
			button.desc = table[index].desc
		else
			if not lastbutton then
				lastbutton = i-1
			end
			button:Hide()
		end
	end
	if not lastbutton then
		lastbutton = numButtons
	end
	parent:SetHeight((lastbutton*20)+10)
	FauxScrollFrame_Update(self, numOptions, numButtons, 20) -- 20 is currently a fixed height for each scroll button
end

local function ButtonOnLeave(self)	
	if not self:GetParent():IsMouseOver(0, 0, 0, 0) then -- GetMouseFocus() maybe???
		self:GetParent():Hide()
	end
end

local function MenuOnEnter(self)
	if self.timer then
		self.timer:Cancel()
		self.timer = nil
	end
end

local function MenuOnLeave(self)
	if not self.trap:IsMouseOver(0, 0, 0, 0) then -- GetMouseFocus() maybe???
		self.timer = C_Timer.NewTimer(0.4, function()
				self:Hide()
			end)
	end
end

local function ScrollButton_Update(self)
	local parent = self:GetParent()
	local parentParent = parent:GetParent()
	local text = ""
	local table = parent.table
	for _, value in pairs(table) do
		if (value.value == self.value) then
			parent:GetParent().setting:SetText(value.text)
			parentParent.data[parentParent.index] = value.value
			if parentParent.updatefunc then
				parentParent.updatefunc(parentParent, parentParent.data, parentParent.index)
			end
			break
		end
	end
	parent:Hide()
end

local function CreateScrollFrame(parent, name, width, height)
	local n, w, h
	n = name or parent:GetName().."_ScrollFrame"
	w = width or 210
	h = height or 410
	local f = CreateFrame("ScrollFrame", n, parent, "FauxScrollFrameTemplate" .. (BackdropTemplateMixin and ", BackdropTemplate" or "")) -- BackdropTemplateMixin required for Classic
	f:SetSize(w, h)
	f:SetBackdropColor(0, 0, 0, 0)
	f:SetBackdropBorderColor(1, 1, 1)
	return f
end

local function CreateScrollButton(parent, id, name, width, height, strata)
	local w, h, s
	w = width or 200
	h = height or 20
	s = strata or "DIALOG"
	local f = CreateFrame("CheckButton", "$parent_ScrollButton"..id, parent)
	f.id = id
	f:SetSize(w, h)
	f:SetFrameStrata(s)
	f:SetHighlightTexture("Interface\\QuestFrame\\UI-QuestTitleHighlight")
	local x = f:GetHighlightTexture()
	x:SetBlendMode("ADD")
	f:SetCheckedTexture(TexturePath.."PlainBackdrop")
	x = f:GetCheckedTexture()
	x:SetBlendMode("ADD")
	f.Text = NS:CreateFont(f, name, "", "TOPLEFT", "TOPLEFT", 0, 0, "LEFT", "CENTER")
	f.Text:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT")
	f.Text:SetShadowColor(0, 0, 0, 1)
	f.Text:SetShadowOffset(1, -1)
	f:SetScript("OnEnter", function(self)
			self:GetParent().timer = nil
			if self.desc then
				GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
				GameTooltip:SetText(self.desc, 1, 1, 1, 1, 1)
			end
		end)
	f:SetScript("OnLeave", function(self)
				GameTooltip:Hide()
				ButtonOnLeave(self)
				
			end)
	f:SetScript("OnClick", function(self)
				ScrollButton_Update(self)
			end)
	return f
end

local function CreateDropMenu(name, parent, label, table, maxdrops, data, index, width, height, labelright) --, override, strata)
	local f = CreateTooltipBackdrop(name, parent, "Button")
	f:SetBackdropColor(0, 0, 0, 0)
	f:SetSize(width, height)
	f:SetFrameStrata("HIGH")
	f.button = CreateFrame("Button", name .. "_Button", f)
	f.button:SetSize(32, 32)
	f.button:ClearAllPoints()
	f.button:SetPoint("RIGHT", f, "RIGHT", 3, -1)
	f.button:SetHitRectInsets(6, 6, 7, 7)
	f.button:SetNormalTexture("Interface\\MainMenuBar\\UI-MainMenu-ScrollDownButton-Up")
	f.button:SetPushedTexture("Interface\\MainMenuBar\\UI-MainMenu-ScrollDownButton-Down")
	f.button:SetDisabledTexture("Interface\\Buttons\\UI-ScrollBar-ScrollDownButton-Disabled")
	f.button:SetHighlightTexture("Interface\\MainMenuBar\\UI-MainMenu-ScrollDownButton-Highlight", "ADD")
	f.button:SetScript("OnClick", function(self)
			DropList_Show(self, true)
		end)
	local left, right, xoffset = "RIGHT", "LEFT", -5
	if labelright then
		left, right, xoffset = "LEFT", "RIGHT", 5
	end
	f.label = NS:CreateFont(f, name, label, left, right, xoffset, 0, "RIGHT", "CENTER")
	f.setting = NS:CreateFont(f, name, "", "LEFT", "LEFT", 3, 0, "LEFT", "CENTER")
	f.setting:ClearAllPoints()
	f.setting:SetPoint("LEFT", f, "LEFT", 5, 0)
	f:SetScript("OnClick", function(self)
			DropList_Show(self)
		end)
	f.initfunc = DropMenu_Init
	f.initfunc(f, data, index, table)
	f.alpha = f:GetAlpha()
	return f
end

local function CreateDropScrollHandler(name, parent, maxdrops, width, height) --, strata)
	local w, h, s
	w = width or 245
	h = (maxdrops*20)+10
	local f = CreateTooltipBackdrop("$parent_DropScrollHandler", parent, "Button")
	f:SetSize(w+20, h+5)
	f:SetFrameStrata(parent:GetFrameStrata())
	f:EnableMouseWheel(false)
	f:SetBackdropColor(0, 0, 0)
	f:SetBackdropBorderColor(.5, .5, .5)
	f.scrollMenu = true
	f.updatefunc = ScrollMenu_Update
	f:Hide()
	f.ScrollFrame = CreateScrollFrame(f, "$parent_ScrollFrame", w, h)
	f.ScrollFrame:SetFrameStrata(parent:GetFrameStrata())
	f.ScrollFrame:SetPoint("TOPLEFT", f, "TOPLEFT")
	f.ScrollFrame:SetBackdropColor(0, 0, 0, 0)
	f.ScrollFrame:SetBackdropBorderColor(1, 1, 1)
	f.ScrollFrame:SetScript("OnVerticalScroll", function(self, offset)
					FauxScrollFrame_OnVerticalScroll(self, offset, 20, self:GetParent().updatefunc)
				end)
	f.Buttons = {}
	for i=1, maxdrops do
		f.Buttons[i] = CreateScrollButton(f, i, "ScrollButton", w, 20, "DIALOG")
		if i == 1 then
			f.Buttons[i]:SetPoint("TOPLEFT", f.ScrollFrame, "TOPLEFT", 10, -5)
		else
			f.Buttons[i]:SetPoint("TOPLEFT", f.Buttons[i-1], "BOTTOMLEFT")
		end

	end
	f.trap = CreateFrame("Frame", "WOOHOO", f)
	f.trap.addheight = parent:GetHeight()+6
	f.trap:SetPoint("CENTER", f, 0, f.trap.addheight/2)
	f.trap:SetFrameLevel(f:GetFrameLevel()-2)
	f:SetScript("OnHide", function(self)
			if self.addedoptions then
				local table = _G[self.table]
				for i in self.addedoptions do
					table[i] = nil
				end
			end
		end)

	f:SetScript("OnEnter", function(self)
			MenuOnEnter(self)
		end)
	f:SetScript("OnLeave", function(self)
			MenuOnLeave(self)
		end)
	f.trap:SetScript("OnEnter", function(self)
			MenuOnEnter(self:GetParent())
		end)
	f.trap:SetScript("OnLeave", function(self)
			MenuOnLeave(self:GetParent())
		end)
	f.ScrollFrame.ScrollBar:HookScript("OnEnter", function(self)
			MenuOnEnter(self:GetParent():GetParent())
		end)
	f.ScrollFrame.ScrollBar:HookScript("OnLeave", function(self)
			MenuOnLeave(self:GetParent():GetParent())
		end)
	return f
end

---------------------------------------------------------------------------------------------------------------------------------------------
function NS:CreateCloseButton(name, parent, width, height)
	local w = width or 24
	local h = height or 24
	local f = CreateFrame("Button", "$parent"..name, parent)
	f:SetSize(w, h)
	f:SetFrameStrata("MEDIUM")
	f:ClearAllPoints()
	f:SetPoint("TOPRIGHT", parent, "TOPRIGHT")--, -2, -4)
	f:SetNormalTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Up")
	f:SetPushedTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Down")
	f:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight", "ADD")
	f:SetScript("OnClick", function(self)
			parent:Hide()
		end)
	return f
end

function NS:CreateFont(frame, name, text, frompoint, topoint, xoffset, yoffset, justifyH, SetJustifyV)
	local x = xoffset or 0
	local y = yoffset or 0
	local f = frame:CreateFontString("$parent"..name)
--	f:ClearAllPoints()
	f:SetFont(FontPath.."Charcoal.ttf", FontSize, "")
	f:SetJustifyH(justifyH or "LEFT")
	f:SetJustifyV(justifyY or "CENTER")
	f:SetTextColor(1, 1, 1, 1)
--	f:SetHeight(10)
	f:SetText(text)
	f:GetWidth()
	f:SetPoint(frompoint, frame, topoint, x, y)
	return f
end

function NS:CreateDefaultButton(name, parent, label, width, height, tooltip)
	local f = CreateFrame("Button", "$parent"..name, parent, "UIPanelButtonTemplate")
	f:SetSize(width, height)
	f:SetText(label)
	if tooltip then
		f:SetScript("OnEnter", function(self)
			GameTooltip:SetOwner(self, "ANCHOR_LEFT")
			GameTooltip:SetText(tooltip)			
		end)
		f:SetScript("OnLeave", function(self)
			GameTooltip:Hide()
		end)
	end
	return f
end

function NS:CreateCheckBox(name, parent, label, data, index, tip, labelleft)
	local f = CreateFrame("CheckButton", "$parent"..name, parent, "UICheckButtonTemplate")
	f:SetSize(CheckSize, CheckSize)
	local left, right, xoffset = "LEFT", "RIGHT", XOfffset
	if labelleft then
		left, right, xoffset = "RIGHT", "LEFT", 0-XOfffset
	end
	f:SetScript("OnClick", function(self)
			local checked = self:GetChecked()
			self.data[index] = checked
			if self.updatefunc then
				self.updatefunc(self, checked)
			end
			NS:UpdateSettings()
		end)
	f:SetScript("OnDisable", function(self)
			self:SetAlpha(0.5)
		end)
	f:SetScript("OnEnable", function(self)
			self:SetAlpha(1)
		end)
	if tip then
		f.tip = tip
		f:SetScript("OnEnter", function(self)
				SetToolTip(self, self.tip)
			end)
		f:SetScript("OnLeave", function(self)
				SetToolTip()
			end)
	end
	f.label = NS:CreateFont(f, "Label", label, left, right, xoffset, 1, right, "CENTER")
	f.initfunc = CheckBox_Init
	f.initfunc(f, data, index)
	return f
end

function NS:CreateSlider(name, parent, label, min, minlock, max, maxlock, step, data, index, width, height, editboxw, editboxh, scale)
	local backdrop = {
		bgFile = "Interface\\Buttons\\UI-SliderBar-Background",
		edgeFile = "Interface\\Buttons\\UI-SliderBar-Border",
		tile = true,
		tileSize = 8,
		edgeSize = 8,
		insets = {
			left = 3,
			right = 3,
			top = 6,
			bottom = 6,
		},
	}
	local editbackdrop = {
		bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
		edgeFile = "Interface\\ChatFrame\\ChatFrameBackground",
		tile = true, 
		edgeSize = 1, 
		tileSize = 5,
	}
	local f = CreateFrame("Slider", "$parent"..name, parent, BackdropTemplateMixin and "BackdropTemplate")
	f:SetOrientation("HORIZONTAL")
	f:SetSize(width, height)
	f:SetHitRectInsets(0, 0, -10, -10)
	f:EnableMouse(true)
	f:SetBackdrop(backdrop)
	f:SetBackdropColor(0, 0, 0, 0)
	f.scale = scale
	f.label = NS:CreateFont(f, name .. "Label", label, "BOTTOM", "TOP", 0, 0, "CENTER", "CENTER")
	f.low = NS:CreateFont(f, name.."Min", min, "TOPLEFT", "BOTTOMLEFT", 2, 2, "LEFT", "CENTER")
	f.high = NS:CreateFont(f, name.."Max", max, "TOPRIGHT", "BOTTOMRIGHT", -2, 2, "RIGHT", "CENTER")
	f:SetThumbTexture("Interface\\Buttons\\UI-SliderBar-Button-Horizontal")
	f.min = min
	f.max = max
	f:SetMinMaxValues(f.min, f.max)
	f.minlock = minlock
	f.maxlock = maxlock
	f:SetValueStep(step)
	f:SetScript("OnValueChanged", function(self)
			Slider_Update(self)
		end)
	f.editbox = CreateFrame("EditBox", "$parent"..name.."EditBox", f, BackdropTemplateMixin and "BackdropTemplate")
	f.editbox:SetFont(FontPath.."Charcoal.ttf", FontSize, "")
	f.editbox:SetAlpha(0.99)
	f.editbox:SetBackdrop(editbackdrop)
	f.editbox:SetBackdropColor(0, 0, 0, 0)
	f.editbox:SetBackdropBorderColor(0.3, 0.3, 0.30, 0.80)
	f.editbox:SetMaxLetters(8)
	f.editbox:SetAutoFocus(false)
	f.editbox:SetJustifyH("CENTER")
	f.editbox:SetFrameStrata("HIGH")
	f.editbox:ClearAllPoints()
	f.editbox:SetPoint("TOP", f, "BOTTOM", 0, 1)
	f.editbox:SetTextColor(1, 1, 0)
	f.editbox:SetScript("OnEscapePressed", function(self)
			self:SetText(self:GetParent():GetValue())
			self:ClearFocus()
		end)
	f.editbox:SetScript("OnEnterPressed", function(self)
			Slider_UpdateFromEditBox(self)
		end)
	function f:GetEditboxWidth()
			return f.editbox:GetWidth()
		end
	function f:GetEditboxHeight()
			return f.editbox:GetHeight()
		end
	function f:SetEditboxSize(width, height)
			f.editbox:SetSize(width, height)
		end
	f:SetEditboxSize(editboxw, editboxh)
	f.initfunc = Slider_Init
	f.initfunc(f, data, index)
	return f
end

function NS:CreateColourPicker(name, parent, label, data, index, labelleft)
	local f = CreateTooltipBackdrop("$parent"..name, parent, "Button")
	f:SetFrameStrata("HIGH")
	f:SetSize(ColourSize, ColourSize)
	local left, right, xoffset = "LEFT", "RIGHT", 5
	if labelleft then
		left, right, xoffset = "RIGHT", "LEFT", -5
	end
	f.label = NS:CreateFont(f, name.."Label", label, left, right, xoffset, 0, right, "CENTER")
	f.initfunc = ColourPicker_Init
	f.initfunc(f, data, index)
	f:SetScript("OnClick", ColourPicker_OnClick)
	return f
end

function NS:CreateSeperator(name, parent, width, height, text)
	local f = CreateFrame("Frame", "$parent"..name, parent)
	f:SetSize(width, height)
	if text then
		f.Texture = f:CreateTexture("$parent_Texture")
		f.Texture:SetAllPoints()
		f.Texture:SetTexture(TexturePath.."PlainBackdrop")
		f.Texture:SetVertexColor(0.3, 0.3, 0.3)
		f.Text = NS:CreateFont(f, text.."Text", text, "CENTER", "CENTER", 0, 0, "CENTER", "CENTER")	
		f.Text:SetTextColor(1, 1, 0)
	end
	return f
end

function NS:CreateEditBox(name, parent, label, width, height, data, index, labelright)
	local f = CreateFrame("EditBox", "$parent"..name, parent, "InputBoxTemplate")
	f:SetSize(width, height)
	f:SetMaxLetters(1000)
	f:SetAutoFocus(false)
	f:SetFrameStrata("HIGH")
	local frompoint, topoint, xoffset, yoffset = "RIGHT", "LEFT", -8, 0
	if labelright then
		frompoint, topoint, xoffset, yoffset = "LEFT", "RIGHT", 5, 0
	end
	f.label = NS:CreateFont(f, "LABEL", label, frompoint, topoint, xoffset, yoffset, "RIGHT", "CENTER")
	f.initfunc = EditBox_Init
	f.initfunc(f, data, index)
	f.writefunc = EditBox_Update
	f:SetScript("OnEscapePressed", function(self)
			if self.prevvalue then
				self:SetText(self.prevvalue)
			else
				self:SetText("")
			end
			self:ClearFocus()
		end)
	f:SetScript("OnEnterPressed", function(self)
			EditBox_Update(self)
		end)
	return f
end
function NS:CreateScrollingDropList(name, parent, label, table, maxdrops, data, index, width, height, labelright) --, override, strata)
	local f = CreateDropMenu(name, parent, label, table, maxdrops, data, index, width, height, labelright) --, override, strata)
	f.menu = CreateDropScrollHandler(name .. "_DropScroll_Handler", f, maxdrops, width, height) --, strata)
	return f
end

function NS:SetPWH(frame, point, anchor, topoint, x, y, width, height, noclear)
	local nclear
	if x == true or y == true or width == true or height == true then
		nclear = true
	end
	if width and width ~= true then frame:SetWidth(width) end
	if height and height ~= true then frame:SetHeight(height or 0) end
	if x == true then x = nil end
	if y == true then y = nil end
	if point then
		if not nclear then
			frame:ClearAllPoints()
		end
		frame:SetPoint(point or "CENTER", anchor or "UIParent", topoint or "CENTER", x or 0, y or 0)
	end
end
